// AD9850 DDS chip driver
// OH2NLT Juha Niinikoski 02.04.2004
// AD9851 test code added 26.08.2004
// Modified for JUMA-TRX2 & Microchip C30 24.11.2660
/*
 The Tuning Word = ( Frequency << 33 ) / ( Reference Oscillator * 6 )
 O/P frequency = Tuning Word * ( Reference Oscillator * 6 ) / 2 ^ 32
 Note that the output frequency is twice the signal frequency, there is a cross-coupled divide by 2 stage
 that is used to generate the I and Q phases.
 
 Example: Crystal Oscillator Frequency = 30MHz
 For an output frequency for 10MHz receive:
 Tuning Word	= 10,000,000 * 2 ^ 33 / 30,000,000 * 6
				= 10,000,000 * 8,589,934,592 / 180,000,000
				= 477,218,583 (Decimal) or 0x1C71C717

 Output Frequency	= 477,218,583 * 180,000,000 / 4,294,967,296
					= 19,999,999.772Hz

 Suppose the 30MHz oscillator is 1kHz high.
 If the displayed frequency is 10MHz, then the actual carrier frequency is:
				= 477,218,583 * 180,006,000 / 4,294,967,296
				= 20,000,666.439Hz

 In order to correct this, the reference oscillator frequency has to be corrected, thus the tuning word is:
				= 10,000,000 * 2 ^ 33 / 30,001,000 * 6
				= 10,000,000 * 8,589,934,592 / 180,006,000
				= 477,202,681 (Decimal) or 0x1C7188F9

 Output frequency	= 477,202,681 * 180,006,000 / 4,294,967,296
					= 19,999,999.971Hz
*/

#define AD9851					// Select DDS type

#include "juma-trx2.h"			// JUMA-TRX2 hardware specific definitions
#include "trx2_eeprom.h"		// JUMA-TRX2 Data EEPROM structures

// Calibration Data
extern union
	{
	struct calval calval;
	unsigned int ee[sizeof(struct calval) / 2];
	} cal;

// Map AD9850 registers to unsigned long 
	struct dds_regs
	{
	unsigned long dds_freq;
	unsigned char dds_config;
	};

	union									// AD9850 frequency & config data, index 0 = LSB
	{
	struct dds_regs dds_regs;
	unsigned char ad9850[5];
	} dds;

// AD9850 data transfer
void dds_send_one(unsigned char data)		// Send one byte LSB first
	{
	int bit_counter = 8;					// Changed to type int from type char - 5B4AIY

	do 	{
		DDS_SER = (data & 0x01);			// This is surely simpler - 5B4AIY
		data >>= 1;							// Move next bit into place
		DDS_W_CLK = 1;						// Generate clock pulse, at a clock rate of 7.5MHz this corresponds
		DDS_W_CLK = 0;						// to a clock pulse width of 133nS, the minimum is 3.5nS - 5B4AIY
		} while (--bit_counter);
	}

void dds_load(unsigned long freq)			// Load AD9850 from buffer
	{
	long long set_val;
	int i = 0;

	set_val = (long long)freq << 33;
	dds.dds_regs.dds_freq = (long)(set_val / (long long)cal.calval.ref_osc);

	do	{
		dds_send_one(dds.ad9850[i]);		// Send 40 bits - Changed i to type int for loop counter - 5B4AIY
		} while (++i < 5);

	DDS_FQ_UD = 1;							// Generate update pulse
	DDS_FQ_UD = 0;
	}

// Init AD9850 interface
void dds_init(void)
	{
	DDS_RESET = 1;							// Reset AD9850
	DDS_SER = 0;
	DDS_RESET = 0;
	DDS_FQ_UD = 1;							// Generate update pulse
	DDS_FQ_UD = 0;							// Set serial mode

#ifdef AD9851	
	dds.dds_regs.dds_config = 0x01;			// Control & phase bits = 0x01, 6x multiplier enabled
#else
	dds.dds_regs.dds_config = 0;			// Control & phase bits = 0
#endif
	}

