// JUMA-TRX2 serial I/O test command set
// Juha Niinikoski, OH2NLT

// Serial test commands moved to own module 03.01.2007

// PA RF attenuator test 23.04.2007
// EEPROM Dump limits increased 08.03.2008
// Bar Graph test 'B' changed to 'b', better PA100 protocol compatibility  15.11.2008

#include <stdio.h>					// Added - 5B4AIY
#include <ctype.h>
#include "juma-trx2.h"				// Board definitions
#include "DataEEPROM.h"				// EEPROM I/O definitions, interface to assembler module
#include "trx2_eeprom.h"			// EEPROM structure definitions

// External references
// External functions
extern void putch(char);						// UART1 I/O
extern int kbhit(void);							// Check for characters in buffer
extern unsigned char getch(void);				// Get a character from the buffer
extern void ClearUSART1queue();					// Flush UART1 buffer
extern void putch2(char);						// UART2 I/O
extern int kbhit2(void);
extern unsigned char getch2(void);
extern void draw_s_meter(int);					// LCD functions
extern void lcd_putst(register const char *);
extern void display_screen(const char *, const char *);
extern void lcd_cmd(unsigned char);
extern void lcdoutch(unsigned char);			// Added - 5B4AIY
extern void set_chgen(int);
extern int convert_adc12(unsigned int);			// ADC
extern void exchg_data_spi1(unsigned int);
extern int set_scale(void);
extern void ms_delay( unsigned int);			// General delay
extern int auto_equalise(int);
extern int cw_speed(int);
extern void display_hdr(void);
extern void save_defval(void);
extern void save_calval(int);

// External Data
extern unsigned char lcdpbuff[];				// LCD print buffer, longer than the display for prints
extern int br_txt[];
extern char *s_meter_type[];
extern char *on_off[];
extern char *mode_msg[];
extern char *agc_speed[];
extern char *audio_input[];
extern char *cw_spd_pot[];
extern char *band_limits[];
extern char *keyer_msg[];
extern char *power_msg[];
extern char *disp_type[];
extern char *cw_rls_speed[];
extern char *Not_Installed[];
extern char *start_page_select[];
extern char *button_type[];
extern char *pwr_mtr_msg[];
extern char *splash_type[];
extern char New_Line[];
extern unsigned char band[];
extern char Equalisation_Aborted[];

extern long freq_in_use;
extern long offset;
extern int isd_1700_installed;
extern int s_meter;
extern int cw_pot;
extern int cw_period;
extern int rit;
extern int rf_filt_idx;				// RF filter & attenuator switch table index
extern unsigned int hc595;			// Main & PA board SPI bus bits
extern int vfo_idx;					// VFO index, A = 0, B = 1, if (Split) then RX = 0, TX = 1
extern int cw_active;				// Status flag for keyer module
extern int PTT_IN;					// Atomic copy of PTT_IN_IO input / TX on, 0 = Active
extern int mute;					// Mute flag
extern int alarm_flag;
extern int loop_ctr;
extern unsigned int ms_counter;

long DDS_freq = 0;

// User Configuration Values
extern union
	{
	struct defval defval;
	} eeprom;

// System Calibration Values
extern union
	{
	struct calval calval;
	} cal;

// Local Data
const char *bandswitch[] = {
							"Default",
							"User (Mode A)",
							"User (Mode B)",
							"Locked"
							};
const char *freq_display[] = {
							"Original\n\r[12.345.68]\n\r[ 2.345.68]\n\r[ 0.345.68]\n\r[ 0.045.68]\n\r[ 0.005.68]\n\r[ 0.000.68]\n\r[ 0.000.08]",
							"New\n\r[12.345678]\n\r[ 2.345678]\n\r[  345.678]\n\r[   45.678]\n\r[    5.678]\n\r[    0.678]\n\r[    0.078]\n\r[    0.008]",
							"Fixed A\n\r[12.345678]\n\r[ 2.345678]\n\r[ 0.345678]\n\r[ 0.045678]\n\r[ 0.005678]\n\r[ 0.000678]\n\r[ 0.000078]\n\r[ 0.000008]",
							"Fixed B\n\r[12.34568 ]\n\r[ 2.34568 ]\n\r[ 0.34568 ]\n\r[ 0.04568 ]\n\r[ 0.00568 ]\n\r[ 0.00068 ]\n\r[ 0.00008 ]"
							};
const char dividing_line[] = {"------------------------------------------\n\r"};
const int filter_freq[] = {
							300,	// 0
							600,	// 1
							1,		// 2
							2,		// 3
							4,		// 4
							8,		// 5
							12,		// 6
							15,		// 7
							19,		// 8
							23,		// 9
							26,		// 10
							28		// 11
							};
const char *vfo_rounding[] = {
							"Off",
							"10Hz",
							"100Hz/10Hz",
							"1kHz/100Hz/10Hz"
							};
const char *two_band_filter[] = {
							"3.5MHz",
							"7.0MHz"
							};
const char *active_filter[] = {
							"Wide",
							"Narrow"
							};
const char *sb_select[] = {
							"USB",
							"LSB"
							};
const char *tx_mode[] = {
							"CW/Tune",
							"SSB"
							};
const char *auto_equalise_msg[] = {
						"Equalisation Complete!",		// 0 - Normal return
						"Power Measurement Error!",		// 1 - Abnormally low output
						"User Initiated Abort!",		// 2 - PTT switch prematurely released
						"Fault Detected!"				// 3 - SWR/Low Voltage/High Voltage/Overcurrent fault
						};

const char lcd_test[] = {"LCD Bar Graph & Character Test,"};
const char any_key_exits[] = {" any key exits."};
const char esc_exits[] = {" ESC exits."};
const char uart1_and_uart2_test[] = {"UART1/UART2 Character Cross-Copy Test,"};
const char write_ASCII[] = {"Write ASCII to LCD,"};
const char trip_fmt[] = {"%4i = %5.2f%c%s"};
const char voltage_alarm[] = {"%s-Voltage Alarm is disabled.%s"};
const char PA_attenuator[] = {" PA Attenuator: %2idB%s"};
const char current_atten_setting[] = {"Current RF Attenuator Setting"};
const char V_A_fmt[] = {"%11.2f%c"};
const char Fwd_Rev_Pwr_fmt[] = {"%3s:%7.2fW"};
const char Seconds[] = {" Seconds"};
const char RIT_fmt[] = {"RIT: %+5iHz"};

void print_filter_freq(int index)
	{
	printf("%4i%cHz", filter_freq[index], (index < 2) ? 'k' : 'M');
	}

void dump_eeprom(void)
	{
	int i;
	unsigned int w;

	printf("%sDump EEPROM contents%s%5s%20s%s%.23s%.22s", New_Line, New_Line, "ADDR", "DATA", New_Line, dividing_line, dividing_line);
	
	for(i = 0; i < 512; i += 2)								// Byte address for EEPROM, but data is stored in words
		{
		ReadEE(EEPAGE, (i + EEDEF), (int *)&w, WORD);		// EEPROM Address 8 high bits, address + physical EEPROM start 16 low bits - Added cast - 5B4AIY

		if((i % 16) == 0) printf("%s%5.4X", New_Line, i);

		printf("%5.4X", w);
		}
	printf("%s%.23s%.22s%s", New_Line, dividing_line, dividing_line, New_Line);
	}

// Only used for Test & Debug
/*
void erase_EEPROM(void)
	{
	int i;

	dump_eeprom();
	
	for(i = 0; i < 512; i += 2) EraseEE(EEPAGE, (i + EEDEF), WORD);

	dump_eeprom();
	save_defval();
	save_calval(1);
	dump_eeprom();
	}
*/
double get_factor(double adc, int cal_factor, double divisor)
	{
	return adc * (double)cal_factor / divisor;
	}

void serial_test(void)
	{
	double f;
	int i, x, temp;
	unsigned int y = 0;
	unsigned char c;

	if(kbhit())
		{
		c = getch();
		switch(c)
			{
// Info commands
			case 'I':
			case '?':
			case 'H':
			case 'h':		// System info
				display_hdr();
				printf("\n\r%.21s[COMMAND TABLE]%.21s%s", dividing_line, dividing_line, New_Line);
				printf("I\tHelp - (This Screen)%s", New_Line);
				printf("A\tDisplay A-D Convertor Samples%s", New_Line);
				printf("a\tAlarm Function Test%s", New_Line);
				printf("b\t%s%s%s", lcd_test, any_key_exits, New_Line);
				printf("C\tClear Factory Default reset counter%s", New_Line);
				printf("c\tContinuous SPI write,%s%s", any_key_exits, New_Line);
				printf("D\t%s%s%s", uart1_and_uart2_test, esc_exits, New_Line);
				printf("E\tDump EEPROM contents%s", New_Line);
				printf("f\tDisplay SPI bus control bits%s", New_Line);
				printf("o\tReference Oscillator calibration value%s", New_Line);
				printf("p\tDisplay scaled S-Meter & CW Speed Pot A-D value%s", New_Line);
				printf("Q\tAuto Equalise RF Output Power%s", New_Line);
				printf("R\tRF Attenuator Settings%s", New_Line);
				printf("S\tStart SCAF (Audio On)%s", New_Line);
				printf("s\tStop SCAF (Audio Off)%s", New_Line);
				printf("t\tSingle SPI write%s", New_Line);
				printf("U\tDump User Frequency Memory%s", New_Line);
				printf("u\tDump Transceiver Configuration Settings%s", New_Line);
				printf("W\t%s%s%s", write_ASCII, esc_exits, New_Line);
				printf("+\tIncrement RF Attenuator%s", New_Line);
				printf("-\tDecrement RF Attenuator%s", New_Line);
				printf("Z\tDivide-By-Zero Trap%s", New_Line);
			break;

// Test commands
// ADC & PWM DAC test
			case 'A':		// ADC Test
				printf("%sA-D Convertor Samples%s Channel   Sample   Scaled(V)   Displayed%s%s", New_Line, New_Line, New_Line, dividing_line);
				x = 9;

				do	{
					temp = convert_adc12(x);
					printf("%5i%11i%11.3f  ", x, temp, get_factor((double)temp, 5000, 4096000.0));

					switch(x)
						{
						case 9:
							printf(V_A_fmt, get_factor((double)temp, cal.calval.id_mult, 2000000.0), 'A');
						break;

						case 10:
							printf(V_A_fmt, get_factor((double)temp, cal.calval.batt_mult, 1000000.0), 'V');
						break;

						case 11:
							i = set_scale();

							if(i > 9) printf("  S9+%2idB", i);
							else printf("     S%1i", i);
						break;

						case 12:
							printf(Fwd_Rev_Pwr_fmt, "Rev", get_factor((double)temp * (double)temp, cal.calval.fwd_pwr_mult, 1000000000.0));
						break;

						case 13:
							printf(Fwd_Rev_Pwr_fmt, "Fwd", get_factor((double)temp * (double)temp, cal.calval.fwd_pwr_mult, 1000000000.0));
						break;

						case 14:
							printf("CW Speed Pot");
						break;

						case 15:
							if(temp >= (2048 + RIT_DEAD_ZONE))	// Blank dead zone
								temp -=(2048 + RIT_DEAD_ZONE);	// Positive side
							else
								{
								if(temp <= (2048 - RIT_DEAD_ZONE))		// Negative side
									temp -=(2048 - RIT_DEAD_ZONE);
								else
									temp = 0;					// Dead zone
								}
							temp >>= 1;							// In decimal frequency system, we divide the A-D value by 2
							printf(RIT_fmt, temp);
						break;
						}
					printf(New_Line);
					} while (++x < 16);
				printf("%s", dividing_line);
			break;

			case 'a':			// Alarm System test
				printf("%sSet Alarm Flag%sEnter 0, 1, 2, 3, 4, 5, or 6 ...%s", New_Line, New_Line, New_Line);
				c = getch();

				switch(c)
					{
					case '0':
						alarm_flag = 0x00;
						printf("%c - All Alarms OFF%s", c, New_Line);
					break;

					case '1':	// If the over-current alarm is enabled...
						if(cal.calval.alarm_flags & 0x01)
							{
							alarm_flag |= 0x01;
							printf("%c - Over-Current Alarm%s", c, New_Line);
							}
						else printf("Over-Current Alarm is disabled.%s", New_Line);
					break;

					case '2':	// High SWR Alarm is always enabled.
						alarm_flag |= 0x02;
						printf("%c - High SWR Alarm%s", c, New_Line);
					break;

					case '3':	// If the Over-Voltage alarm is enabled...
						if(cal.calval.alarm_flags & 0x04)
							{
							alarm_flag |= 0x04;
							printf("%c - Over-Voltage Alarm%s", c, New_Line);
							}
						else printf(voltage_alarm, "Over", New_Line);
					break;

					case '4':	// If the Under-Voltage alarm is enabled...
						if(cal.calval.alarm_flags & 0x08)
							{
							alarm_flag |= 0x08;
							printf("%c - Under-Voltage Alarm%s", c, New_Line);
							}
						else
							printf(voltage_alarm, "Under", New_Line);
					break;

					case '5':	// If the Under-Voltage alarm is enabled...
						if(cal.calval.alarm_flags & 0x08)
							{
							alarm_flag |= 0x10;
							printf("%c - Low Battery (Under-Voltage Pre-Limit) Alarm%s", c, New_Line);
							}
						else
							printf(voltage_alarm, "Under", New_Line);
					break;

					case '6':
						alarm_flag = 0x1F;
						printf("%c - All Alarms ON%s", c, New_Line);
					break;

					default:
					break;
					}

				loop_ctr = 0;	// Reset alarm blink counter
			break;
			
			case 'b':			// Bar graph test
				temp = eeprom.defval.S_Meter_Type;
				eeprom.defval.S_Meter_Type = GRAPHIC;
				printf("%s%s%s%sSelect Scale: 0,1,2: ", New_Line, lcd_test, any_key_exits, New_Line);
				c = (getch() - '0') % 3;
				printf("%d - %s%s", c, disp_type[c], New_Line);
				set_chgen(c);					// Select the desired character generator map
				c = 0;							// Initialise the character
				display_screen("", "[      ]");

				do	{							// The maximum value for the S-meter is 48.
					x = 0;

					do	{
						lcd_cmd(LINE1);
						draw_s_meter(x);
						ms_delay(10);
						} while (++x < 49);

					lcd_cmd(LINE2PLUS1);
					lcdoutch(c);
					sprintf((char *)lcdpbuff, " =%3.2X", c);	// Display LCD characters - Added cast - 5B4AIY
					lcd_putst((char *)lcdpbuff);
					c++;
					ms_delay(500);
					} while(!kbhit());

				ClearUSART1queue();						// Consume the break character by flushing the buffer
				eeprom.defval.S_Meter_Type = temp;		// Restore S-Meter type
				set_chgen(eeprom.defval.Display_Type);	// Load the original bar graph fonts
			break;

			case 'C':		// Clear factory default reset counter
				printf("%sEEPROM Reset Counter cleared.", New_Line);
				temp = 0;
				EraseEE(EEPAGE, EE_FD_LOC, WORD);
				WriteEE(&temp, EEPAGE, EE_FD_LOC, WORD);
				dump_eeprom();
			break;

			case 'c':		// Continous test write SPI bus
				printf("%sContinuous SPI Transmit,%s%s", New_Line, any_key_exits, New_Line);

				do	{
					y++;
					exchg_data_spi1(y);
					ms_delay(10);

					if(kbhit())				// Check for a character
						{
						ClearUSART1queue();	// Flush keyboard buffer
						break;
						}
					} while(TRUE);
				printf(New_Line);
			break;

			case 'D':		// Debug RS232 input, cross copy
				printf("%s%s%s%s", New_Line, uart1_and_uart2_test, esc_exits, New_Line);
				clear_lcd();

				do	{
					putch2(c = getch());
					lcd_cmd(LINE1);		
					lcdoutch(c);

					if(kbhit2())
						{
						putch(c = getch2());
						lcd_cmd(LINE2);
						lcdoutch(c);
						}
					} while(c != 0x1B);	// Exit with ESC

				printf("%s Terminated.%s", uart1_and_uart2_test, New_Line);
			break;

			case 'd':		// Test, debug print some internal variables
				printf("%sSpecial Internal Variable Dump", New_Line);
				printf("%seeprom.defval.max_vfo = %i", New_Line, eeprom.defval.max_vfo);
				printf("%seeprom.defval.vfo_sel = %i", New_Line, eeprom.defval.vfo_sel);
				printf("%svfo_idx = %i", New_Line, vfo_idx);
				printf("%seeprom.defval.mode[vfo_idx] = %i", New_Line, eeprom.defval.mode[vfo_idx]);
				printf("%seeprom.defval.mode[2] = %i", New_Line, eeprom.defval.mode[2]);
				printf("%scw_active = %i", New_Line, cw_active);
				printf("%sPTT_IN = %i", New_Line, PTT_IN);
				printf("%sfreq_in_use  :%9ld", New_Line, freq_in_use);
				printf("%sDDS Frequency:%9ld", New_Line, (freq_in_use + offset));
				printf("%sOffset       :%+9ld", New_Line, offset);
				printf("%sRIT          :%+9d", New_Line, rit);
				printf("%smSec Counter = %i%s", New_Line, ms_counter, New_Line);
			break;

			case 'E':		// Dump EEPROM
				dump_eeprom();
			break;
/*
			case 'e':		// Only used for Test & Debug
				erase_EEPROM();
			break;
*/
			case 'f':		// Print RF Attenuator setting & SPI bus bits
				printf("%sFully Decoded SPI Bus Data%s%.26s%s", New_Line, New_Line, dividing_line, New_Line);
				printf("Data            : 0x%4.4X%s", hc595, New_Line);
				printf("Attenuator      : %ddB%s", ((hc595 & 0xC000) >> 14), New_Line);
				printf("Spare Bits      : %d%s", ((hc595 & 0x3800) >> 11), New_Line);
				printf("Filter Bits     : %d%sFrequency       : ", ((hc595 & 0x0700) >> 8), New_Line);
				print_filter_freq(rf_filt_idx);
				printf("%sDual-Band Filter: %s%s", New_Line, two_band_filter[((hc595 & 0x0080) >> 7)], New_Line);
				printf("Audio Source    : %s%s", audio_input[((hc595 & 0x0040) >> 6)], New_Line);
				printf("Noise Blanker   : %s%s", on_off[((hc595 & 0x0020) >> 5)], New_Line);
				printf("Speech Processor: %s%s", on_off[((hc595 & 0x0010) >> 4)], New_Line);
				printf("AGC Speed       : %s%s", agc_speed[((hc595 & 0x0008) >> 3)], New_Line);
				printf("Active Filter   : %s%s", active_filter[((hc595 & 0x0004) >> 2)], New_Line);
				printf("Sideband        : %s%s", sb_select[((hc595 & 0x0002) >> 1)], New_Line);
				printf("Mode            : %s%s%.26s%s", tx_mode[(hc595 & 0x0001)], New_Line, dividing_line, New_Line);
			break;

			case 'o':		// Test, print ref osc calibration value
				printf("%sRef Osc = %li MHz%s", New_Line, cal.calval.ref_osc, New_Line);
			break;

			case 'p':		// S-meter, CW/Squelch pot, equivalent CW speed in wpm
				printf("%sS-Meter = %2i POT = %4i WPM = %2i%s", New_Line, s_meter, cw_pot, 1200 / cw_speed(cw_pot), New_Line);  // Fixed bug - 5B4AIY
			break;

			case 'Q':		// Auto-Equalise RF O/P power
				printf("%sAuto Equalise RF Output Power%s", New_Line, New_Line);
				printf("Ensure transceiver is connected to a 50 Ohm dummy load.%s", New_Line);
				printf("This may alter your current attenuator settings.%s", New_Line);
				printf("Do you wish to continue (Y/N)?: ");
				putch(c = toupper(getch()));
				printf(New_Line);
				x = NORMAL_EXIT;

				if(c == 'Y')
					{
					x = auto_equalise(TRUE) & 0x00FF;		// Mask off the reference band data
					printf("%s%s", auto_equalise_msg[x], New_Line);
					}

				if(x) printf("Equalisation Aborted - No changes made.%s", New_Line);
			break;

			case 'R':		// Display RF Attenuator Settings
				printf("%sRF Attenuator Settings%s Filter  Attenuator%s%.20s%s", New_Line, New_Line, New_Line, dividing_line, New_Line);
				i = 11;

				do	{
					print_filter_freq(i);
					printf("%7idB%s", cal.calval.rf_atn_bits[i], New_Line);
					} while (i--);

				printf("%.20s%s", dividing_line, New_Line);
			break;

			case 's':		// Stop SCAF, Audio Off
				printf("%sMute%s", New_Line, New_Line);
				mute = TRUE;
			break;

			case 'S':		// Start SCAF, Audio On
				printf("%sAudio On%s", New_Line, New_Line);
				mute = FALSE;
			break;

			case 't':		// Test write SPI bus
				printf("%sSPI Tx%s", New_Line, New_Line);
				exchg_data_spi1(0x5501);
			break;

			case 'U':		// Display VFO Memory Settings
				printf("%s  VFO Memory Settings%s%.23s%s", New_Line, New_Line, dividing_line, New_Line);

				for(i = 0, c = 'A'; i < 26; i++, c++)
					{
					f = (double)eeprom.defval.vfo[i];

					if(f < 1000000.0) printf("VFO-%c: %8.3f kHz", c, f / 1000.0);
					else printf("VFO-%c: %8.5f MHz", c, f / 1000000.0);

					printf(" %s%s", mode_msg[eeprom.defval.mode[i]], New_Line);
					}

				printf("%sUser Frequency Settings%s%.23s%s", New_Line, New_Line, dividing_line, New_Line);

				for(i = 30, c = 1; i < 39; i++, c++)
					printf("%4im: %8.5f MHz %s%s", band[c], (double)eeprom.defval.vfo[i] / 1000000.0, mode_msg[eeprom.defval.mode[i]], New_Line);

				printf("%.23s%s", dividing_line, New_Line);
			break;

			case 'u':		// Dump Transceiver configuration settings
// System Settings, Firmware Version & Build
				printf("%s%25s%s", New_Line, "System Settings", New_Line);
				printf("     Firmware: %s Build: %2s%s%.36s%s", VERSION, BUILD, New_Line, dividing_line, New_Line);
				printf("Reference Oscillator: %9li Hz%s", cal.calval.ref_osc, New_Line);
				printf("DC Voltmeter        : %4d%s", cal.calval.batt_mult, New_Line);
				printf("Ammeter             : %4d%s", cal.calval.id_mult, New_Line);
				printf("Power Meter         : %4d%s", cal.calval.fwd_pwr_mult, New_Line);
				printf("S-Meter             : %-d%s", cal.calval.s_mtr_cal, New_Line);

				if(cal.calval.beep_len)
					printf("Beep Time           : %-d mS%s", cal.calval.beep_len, New_Line);
				else
					printf("Beep                : OFF%s", New_Line);

				printf("SWR Trip Setting    : %-.2f:1%s", ((double)cal.calval.swr_trip) / 100.0, New_Line);
				printf("SWR Long Delay Time :");

				if(cal.calval.swr_delay)
					printf(" %-.1f Secs%s", (double)cal.calval.swr_delay, New_Line);
				else
					printf(" Disabled%s", New_Line);

				printf("Over-Current Trip   : ");

				if(cal.calval.alarm_flags & 0x01)
					printf(trip_fmt, cal.calval.overcurrent_trip, get_factor((double)cal.calval.overcurrent_trip, cal.calval.id_mult, 2000000.0), 'A', New_Line);
				else
					printf("%s%s", on_off[0], New_Line);

				printf("Over-Voltage Trip   : ");

				if(cal.calval.alarm_flags & 0x04)
					printf(trip_fmt, cal.calval.overvoltage_trip, get_factor((double)cal.calval.overvoltage_trip, cal.calval.batt_mult, 1000000.0), 'V', New_Line);
				else
					printf("%s%s", on_off[0], New_Line);

				printf("Under-Voltage Trip  : ");

				if(cal.calval.alarm_flags & 0x08)
					printf(trip_fmt, cal.calval.undervoltage_trip, get_factor((double)cal.calval.undervoltage_trip, cal.calval.batt_mult, 1000000.0), 'V', New_Line);
				else
					printf("%s%s", on_off[0], New_Line);

				printf("Long Button Mode    : %s%s", button_type[hold_latch], New_Line);
				printf("Band Limits         :%s%s", band_limits[cal.calval.Band_Limits], New_Line);
				printf("Splash Screen       : %s%s", splash_type[cal.calval.splash], New_Line);
// User Configuration Settings
				printf("%s%24s%s%.36s%s", New_Line, "User Settings", New_Line, dividing_line, New_Line);
				printf("AGC Speed        : %s%s", agc_speed[eeprom.defval.agc_speed], New_Line);
				printf("Filter Roll-Off Frequencies%s", New_Line);
				printf("Narrow           : %4li Hz%s", 75000L / eeprom.defval.scaf_filters[0], New_Line);
				printf("Mid              : %4li Hz%s", 75000L / eeprom.defval.scaf_filters[1], New_Line);
				printf("Wide             : %4li Hz%s", 75000L / eeprom.defval.scaf_filters[2], New_Line);
				printf("Transmit         : %4li Hz%s", 75000L / eeprom.defval.scaf_filters[3], New_Line);
				printf("Speech Processor : %3s%s", on_off[eeprom.defval.proc_on_off], New_Line);
				printf("Audio Source     : %s%s", audio_input[eeprom.defval.af_input], New_Line);
				printf("Keyer Mode       : %8s%s", keyer_msg[eeprom.defval.keyer], New_Line);
				printf("CW Release Delay : ");

				if(eeprom.defval.CW_Release > MIN_CW_RELEASE)
					printf("%-d mS", eeprom.defval.CW_Release);
				else
					printf("%s", cw_rls_speed[eeprom.defval.sq]);

				printf("\n\rCW Pitch         : %-d Hz%s", eeprom.defval.cw_shift, New_Line);
				printf("CW Pot Mode      : %s%s", cw_spd_pot[eeprom.defval.sq], New_Line);
				temp = cw_speed(eeprom.defval.CW_Speed & UNLOCKED);
				printf("Keyer Speed      : ");

				if(eeprom.defval.CW_Speed & LOCKED)
					printf("Locked%5d wpm%s", (1200 / temp), New_Line);
				else
					printf("Unlocked%3d wpm%s", (1200 / temp), New_Line);

				printf("CW/CWR Swap      : %s%s", on_off[eeprom.defval.CW_CWR_Swap], New_Line);
				printf("LCD Backlight    : %-d%s", eeprom.defval.back_light, New_Line);
				printf("LCD Contrast     : %-d%s", eeprom.defval.contrast, New_Line);
				printf("LCD Timer        : %-d%s", eeprom.defval.back_light_timer, Seconds);

				if(!eeprom.defval.back_light_timer)
					printf(" (Off)");

				printf("%sRS-232 Mode      : Test%s", New_Line, New_Line);
				printf("Speed            : %-d00 Baud%s", br_txt[eeprom.defval.br], New_Line);
				printf("VFO Memory       : ");

				if(eeprom.defval.max_vfo == 2)
					printf("A/B + Split");
				else
					printf("%2i Memories", eeprom.defval.max_vfo);

				printf("%sS-Meter          : %7s%s", New_Line, s_meter_type[eeprom.defval.S_Meter_Type], New_Line);
				printf("S-Meter Hold Time: %-.2f%s%s", ((double)eeprom.defval.Hold_Time) / 1000.0, Seconds, New_Line);
				printf("TX Disable       : %3s%s", on_off[eeprom.defval.OOB_Check], New_Line);
				printf("Auto-Sideband    : %3s%s", on_off[eeprom.defval.Auto_SB], New_Line);
				printf("Frequency Display: %s%s", freq_display[eeprom.defval.Freq_Display], New_Line);
				printf("Band Switch      : %s%s", bandswitch[eeprom.defval.Band_SW], New_Line);
				printf("Include RIT      : %s%s", on_off[eeprom.defval.RIT_Include], New_Line);
				printf("Tune Attenuator  :%3idB%s", -eeprom.defval.Tune_Attenuation, New_Line);
				printf("VFO Adjustment   : ");

				if(eeprom.defval.VFO_Adjust)
					printf("%s decade(s)", vfo_rounding[eeprom.defval.VFO_Adjust]);
				else
					printf("Off");

				printf("%sMeter Scale Type : %s%s", New_Line, disp_type[eeprom.defval.Display_Type], New_Line);
				printf("Power-Off Mode   : %8s%s", power_msg[eeprom.defval.nb], New_Line);
				printf("Power Meter Type : %s%s", pwr_mtr_msg[eeprom.defval.Power_Meter], New_Line);
				printf("Start-Up Page    : %s", start_page_select[display_page & 0x07]);

				if((display_page & 0x07) == LAST_PAGE)
					printf(" - %s", start_page_select[display_page >> 4]);

				printf("%sVoice Memory     : %sInstalled%s", New_Line, Not_Installed[isd_1700_installed], New_Line);
				printf("%.36s%s", dividing_line, New_Line);
			break;

			case 'W':		// Test LCD character generator, ASCII
				printf("%s%s%s%s", New_Line, write_ASCII, esc_exits, New_Line);
				clear_lcd();
				x = 0;

				do	{
					c = getch();

					if(x == 16) lcd_cmd(LINE2);
					if(c == 0x0D || x == 32)
						{
						clear_lcd();
						x = 0;
						}

					if(isprint(c))
						{
						printf("Char = 0x%.2X\n\r", c);
						lcdoutch(c);
						x++;
						}
					else printf("Char = 0x%.2X - Non-Printable Character\n\r", c);
					} while(c != 0x1B);

				printf("Test Terminated.\n\r");
			break;

			case '+':		// Increment/Decrement Current Band Attenuator
			case '-':
				printf("%s%screment %s%s", New_Line, (c == '+') ? "In" : "De", current_atten_setting, New_Line);
				(c == '+') ? cal.calval.rf_atn_bits[rf_filt_idx]++ : cal.calval.rf_atn_bits[rf_filt_idx]--;
				cal.calval.rf_atn_bits[rf_filt_idx] %= 4;
				printf("Filter:");
				print_filter_freq(rf_filt_idx);
				printf(PA_attenuator, cal.calval.rf_atn_bits[rf_filt_idx], New_Line);
			break;

			case 'Z':		// Trap / Error test
				printf("%sDivide By Zero%s", New_Line, New_Line);
				x = 0;
				temp = temp / x;			// Generate error ==> trap
			break;

			default:
				printf("%sInvalid Command: %c Value: %d%s", New_Line, c, c, New_Line);
				ClearUSART1queue();			// Flush the buffer.
			break;
			}	// case
		}	// if
	}

